
import AnimatedBox;

import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.Graphics;
import java.net.URL;
import java.util.Vector;
import java.util.Enumeration;

public class Bounce extends AnimationShell
	{
	/*
	This applet displays some boxes that bounce around inside other boxes.
	It takes numBoxes and soundFile<i> as parameters from the html page.
	The animation display code is in the superclass, and the animation itself
	is generated by the interaction of several AnimatedBoxes.

	This class relies on its superclass to provide all the normal runnable
	applet methods: init(), start(), stop(), and run(), and it relies on
	each AnimatedBox to understand how to paint() itself, changeState(),
	reproduce(), and getRelatives(). It's job is to interface between these
	two classes and the user interface.

	Methods:
	--------
	init()                (first method invoked by browser or appletviewer)
		fetchSounds()      fetch any bounce sounds named in the html page
		fetchNumBoxes()    fetch the number of boxes from the html page
		broadcastBoxes()   pass the vector of boxes to each of the boxes
	updateAnimation()     called by the run() method of the superclass
	paintAnimationFrame() called by the paint() method of the superclass
	*/

	//the bouncing boxes to animate
	Vector boxes = new Vector();

	//the default number of boxes to create
	int numBoxes = 2;

	////////////////////////////////////////////////////////////////////
	public void init()
		{
		/*
		Initialize the applet.
		*/

		super.init();

		//create the first (largest, outermost, and stationary) box
		AnimatedBox firstBox =
			new AnimatedBox(1, 1, super.windowWidth - 2, super.windowHeight - 2,
								fetchSounds());

		//add it to the (currently empty) vector of boxes
		boxes.addElement(firstBox);

		//fetch numBoxes and tell the first box to spawn itself numBoxes times
		//(which also fills the boxes vector)
		boxes = firstBox.reproduce(fetchNumBoxes());

		//pass the filled vector of boxes to all of the boxes just created
		broadcastBoxes();
		}

	////////////////////////////////////////////////////////////////////
	public void updateAnimation()
		{
		/*
		Tell each AnimatedBox to change state for one clock tick.

		Note: This method overrides an empty method in my superclass;
		it is regularly called by my superclass.
		*/

		Enumeration list = boxes.elements();
		while (list.hasMoreElements())
			((AnimatedBox) list.nextElement()).changeState();
		}

	////////////////////////////////////////////////////////////////////
	public void paintAnimationFrame(Graphics graphicsContext)
		{
		/*
		Tell each AnimatedBox to paint itself into graphicsContext.

		Note: This method overrides an empty method in my superclass; it is
		called by the paint() method of my superclass, which actually controls
		painting to the screen. My superclass passes graphicsContext here and
		i pass that graphicsContext to each AnimatedBox, which then paints
		itself into graphicsContext.
		*/

		Enumeration list = boxes.elements();
		while (list.hasMoreElements())
			((AnimatedBox) list.nextElement()).paint(graphicsContext);
		}

	////////////////////////////////////////////////////////////////////
	protected Vector fetchSounds()
		{
		/*
		Return a Vector of any bounce sound files by querying my AppletContext
		(which will try to read them from the html page, if they're set there).
		Look for them in the same directory as this applet.
		*/

		Vector sounds = new Vector();
		int numSoundsFetched = 0;
		URL tempUrl; String tempString; AudioClip tempSound;
		while (true)
			{
			try
				{
				tempUrl = this.getCodeBase();
				tempString = this.getParameter("soundFile" + (numSoundsFetched + 1));
				tempSound = this.getAudioClip(tempUrl, tempString);
				if (tempSound == null)
					break;
				else
					{
					sounds.addElement(tempSound);
					numSoundsFetched++;
					}
				}
			catch (Exception ignored)
				{ break; } //if anything goes wrong, simply fail
			}

		if (numSoundsFetched <= 0)
			System.out.println("Couldn't find sound files " +
				"from parameter `soundFile1'; using no sound.");

		return sounds;
		}

	////////////////////////////////////////////////////////////////////
	protected int fetchNumBoxes()
		{
		/*
		Fetch the number of boxes parameter from my AppletContext
		(which will try to read it from the html page, if it's set there).
		Use the default if the number is absent, zero or less, or malformed.
		*/

		int tempNumber = 0;
		String tempString = this.getParameter("numBoxes");
		if (tempString != null)
			{
			try {tempNumber = Integer.parseInt(tempString);}
			catch (NumberFormatException ignored)
				{
				System.out.println("Couldn't parse parameter `numBoxes'; " +
				"using default instead.");
				}
			}

		if (tempNumber <= 0)
			tempNumber = numBoxes;

		return tempNumber;
		}

	////////////////////////////////////////////////////////////////////
	protected void broadcastBoxes()
		{
		/*
		Tell each AnimatedBox about all the other AnimatedBoxes
		by passing the whole vector of boxes to each of them.
		*/

		Enumeration list = boxes.elements();
		while (list.hasMoreElements())
			((AnimatedBox) list.nextElement()).getRelatives(boxes);
		}
	}
