
import java.awt.*;
import java.applet.*;
import java.net.*;
import java.util.*;
import java.io.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class DummyAppletContext extends Frame
	implements AppletStub, AppletContext, URLStreamHandlerFactory
	//ActionListener
	{
	/*
	Objects of this class let any applet run as an application without
	change. The only thing it can't (yet) do is access remote url's.

	AppletContext is the applet's environment---normally the browser plus
	the applet's enclosing html file. AppletStub is used by the Applet
	superclass to help implement applet functions like getAppletContext() and
	getParameter().
	*/

	//the list of all applets sharing this context
	private Vector appletList;

	//the name-value pair list of applet parameters
	private Hashtable appletParameters = new Hashtable();

	//the applet status bar
	private TextField appletStatusBar;

	//the applet's window dimensions
	private int initialWindowWidth, initialWindowHeight;

	//////////////////////////////////////////////////////////////////////
	public static void main(String[] commandLineArguments)
		{
		/*
		This method makes it unnecessary to add a main() method to the applet
		just to run it as a stand-alone application.
		*/

		new DummyAppletContext(commandLineArguments);
		}

	//////////////////////////////////////////////////////////////////////
	public DummyAppletContext(Applet applet, String[] commandLineArguments)
		{
		/*
		Construct an AppletContext for a preexisting applet.
		*/

		this(applet, 640, 480, commandLineArguments);
		}

	//////////////////////////////////////////////////////////////////////
	public DummyAppletContext(Applet applet, int windowWidth, int windowHeight,
										String[] commandLineArguments)
		{
		/*
		Construct an AppletContext for a preexisting applet with a given
		window width and height.
		*/

		super(applet.getClass().getName());

		init(applet, windowWidth, windowHeight, commandLineArguments, 0);
		}

	//////////////////////////////////////////////////////////////////////
	public DummyAppletContext(String[] commandLineArguments)
		{
		/*
		Construct an AppletContext. commandLineArguments contains the applet's
		name, it's requested height and width (if present), and any other
		parameters it needs as [name value] pairs.
		*/

		super(commandLineArguments[0]);

		try
			{
			//try to create an instance of the applet subclass
			Applet applet =
				(Applet) Class.forName(commandLineArguments[0]).newInstance();

			init(applet, 640, 480, commandLineArguments, 1);
			}
		catch (Exception exception)
			{
			//can't do anything here; any exception is fatal, so simply die
			exception.printStackTrace();
			System.exit(1);
			}
		}

	//////////////////////////////////////////////////////////////////////
	private void init(Applet applet, int defaultWidth, int defaultHeight,
							String[] commandLineArguments, int startIndex)
		{
		/*
		Initialization code shared by all the above constructors.

		startIndex is the index into commandLineArguments at which to start
		parsing parameters
		*/

		//register myself as the applet's URLStreamhandler
		URL.setURLStreamHandlerFactory(this);

		//put the applet in a vector (to be returned later by getApplets())
		appletList = new Vector();
		appletList.addElement(applet);

		//tell the applet that i will be its AppletStub
		applet.setStub(this);

		initialWindowWidth = defaultWidth; initialWindowHeight = defaultHeight;

		parseArgs(commandLineArguments, startIndex);

		//add the applet to my window
		add("Center", applet);

		//add the applet's status bar
		appletStatusBar = new TextField();
		appletStatusBar.setEditable(false);
		add("South", appletStatusBar);

		appletResize(initialWindowWidth, initialWindowHeight);

		//show my window, then call the applet's init() and start() methods
		show();
		applet.init();
		applet.start();
		}

	//////////////////////////////////////////////////////////////////////
	public void parseArgs(String commandLineArguments[], int startIndex)
		{
		/*
		Parse the command line arguments into parameters and add any parameters
		found to the appletParameters hashtable. Get the requested window width
		and height if specified (-width [value] -height [value]), and any applet
		parameters (as [name value] pairs) that may be given.

		startIndex is the index in commandLineArguments at which to start parsing
		*/

		int endIndex = commandLineArguments.length - startIndex;
		if ((endIndex % 2) != 0)
			{
			System.err.println("Warning: odd number of parameters supplied. " +
									"Ignoring all supplied parameters.");
			return;
			}

		for (int index = startIndex; index < endIndex; index += 2)
			{
			String currentArgument = commandLineArguments[index];
			String nextArgument = commandLineArguments[index + 1];
			try
				{
				if (currentArgument.equals("-width"))
					initialWindowWidth = Integer.parseInt(nextArgument);

				if (currentArgument.equals("-height"))
					initialWindowHeight = Integer.parseInt(nextArgument);

				if (! ((currentArgument.equals("-height")) ||
					(currentArgument.equals("-width"))))
					appletParameters.put(currentArgument, nextArgument);
				}
			catch (NumberFormatException exception)
				{
				System.err.println("Warning: couldn't parse parameter " +
										currentArgument + ". Ignoring remainder.");
				}
			}
		}

	//////////////////////////////////////////////////////////////////////
	public boolean handleEvent(Event event)
		{
		/*
		Event handler to catch the Frame (window) close action,
		and exit the program.
		*/

		if (event.id == Event.WINDOW_DESTROY)
			System.exit(0);

		return super.handleEvent(event);
		}

/************ AppletStub methods *************/

	//////////////////////////////////////////////////////////////////////
	public boolean isActive()
		{
		/*
		Return true if the applet is active (so always return true).
		*/

		return true;
		}

	//////////////////////////////////////////////////////////////////////
	public URL getDocumentBase()
		{
		/*
		Return the document URL. (actually return a "file:" URL for the current
		directory)
		*/

		URL url = null;
		try
			{
			File dummy = new File("dummy.html");
			String path = dummy.getAbsolutePath();
			if (! File.separator.equals("/"))
				{
				StringBuffer buffer = new StringBuffer();
				if (path.charAt(0) != File.separator.charAt(0))
					buffer.append("/");

				StringTokenizer st = new StringTokenizer(path, File.separator);
				while (st.hasMoreTokens())
					buffer.append(st.nextToken() + "/");

				if (File.separator.equals("\\") &&
					(buffer.charAt(2) == ':'))
					buffer.setCharAt(2, '|');

				path = buffer.toString();
				path = path.substring(0, path.length() - 1);
				}
			url = new URL("file", "", -1, path);
			}
		catch (MalformedURLException exception)
			{
			exception.printStackTrace();
			}

		return url;
		}

	//////////////////////////////////////////////////////////////////////
	public final URL getCodeBase()
		{
		/*
		Return the codebase URL. (actually return the same value as
		getDocumentBase())
		*/

		return getDocumentBase();
		}

	//////////////////////////////////////////////////////////////////////
	public final String getParameter(String parameterName)
		{
		/*
		Return one of the applet's parameters.
		*/

		return (String) appletParameters.get(parameterName);
		}

	//////////////////////////////////////////////////////////////////////
	public final AppletContext getAppletContext()
		{
		/*
		Return a handler to the applet's context.
		*/

		return (AppletContext) this;
		}

	//////////////////////////////////////////////////////////////////////
	public void appletResize(int windowWidth, int windowHeight)
		{
		/*
		Resize myself to fit the applet's request to be resized.
		*/

		Insets insets = getInsets();

		this.setSize((windowWidth + insets.left + insets.right),
						(windowHeight + insets.top + insets.bottom +
						appletStatusBar.getPreferredSize().height));
		}

/************ AppletContext methods *************/

	//////////////////////////////////////////////////////////////////////
	public final AudioClip getAudioClip(URL url)
		{
		/*
		* Gets an audio clip. (There doesn't seem to be a "Toolkit" for
		* audio clips in my JDK, so this always returns null. You could
		* implement this differently, returning a dummy AudioClip object
		* for which the class could be defined at the bottom of this file.)
		*
		* @param url URL of the AudioClip to load
		* @return the AudioClip object if it exists (in our case,
		* this is always null
		*/

		return null;
		}

	//////////////////////////////////////////////////////////////////////
	public final Image getImage(URL url)
		{
		/*
		* Gets an image. This usually involves downloading it
		* over the net. However, the environment may decide to
		* cache images. This method takes an array of URLs,
		* each of which will be tried until the image is found.
		*
		* @param url URL of the Image to load
		* @return the Image object
		*/

		return Toolkit.getDefaultToolkit().getImage(filenameFromURL(url));
		}

	//////////////////////////////////////////////////////////////////////
	private String filenameFromURL(URL url)
		{
		/*
		PRIVATE utility function. Ignores the protocol, and returns a
		filename for a file on the local filesystem (which may or may
		not exist).
		*/

		String filename = url.getFile();
		if (filename.charAt(1) == '|')
			{
			StringBuffer buf = new StringBuffer(filename);
			buf.setCharAt(1, ':');
			filename = buf.toString();
			}
		else
			if (filename.charAt(2) == '|')
				{
				StringBuffer buf = new StringBuffer(filename);
				buf.setCharAt(2, ':');
				filename = buf.toString();
				}

		return filename;
		}

	//////////////////////////////////////////////////////////////////////
	public final Applet getApplet(String appletName)
		{
		/*
		Return an applet given its name. (return null if the applet does not
		exist, and it never can here since we never name the applet).
		*/

		return null;
		}

	//////////////////////////////////////////////////////////////////////
	public final Enumeration getApplets()
		{
		/*
		Return an enumeration of the applets in this context. (in this case,
		the enumeration contains only the applet created with this
		DummyAppletContext)
		*/

		return appletList.elements();
		}

	//////////////////////////////////////////////////////////////////////
	public void showDocument(URL url)
		{
		/*
		* Shows a new document. This may be ignored by
		* the applet context (and in our case, it is, but we'll show the
		* user, in the status area, that the document was requested and
		* WOULD be loaded if in a browser).
		*
		* @param url URL to load
		*/

		appletStatusBar.setText("AppletContext request to show URL " + url);
		}

	//////////////////////////////////////////////////////////////////////
	public void showDocument(URL url, String target)
		{
		/*
		* Show a new document in a target window or frame. This may be ignored by
		* the applet context. (Again, it is ignored, but we'll show the
		* request information to the user in the status area.)
		*
		* This method accepts the target strings:
		* _self		show in current frame
		* _parent	show in parent frame
		* _top		show in top-most frame
		* _blank		show in new unnamed top-level window
		* <other>	show in new top-level window named <other>
		*
		* @param url URL to load
		* @param target the target string
		*/

		appletStatusBar.setText("AppletContext request to show URL " + url +
										" in target: " + target);
		}

	//////////////////////////////////////////////////////////////////////
	public void showStatus(String string)
		{
		/*
		Display a string in the status area.
		*/

		appletStatusBar.setText(string);
		}

/************ URLStreamHandlerFactory methods *************/

	//////////////////////////////////////////////////////////////////////
	public URLStreamHandler createURLStreamHandler(String protocol)
		{
		/*
		Creates a new URLStreamHandler instance with the specified protocol.
		(ftp, http, nntp, etc.).

		Note: THIS PROTOCOL IS IGNORED BY THIS APPLET CONTEXT
		*/

		return new DummyURLStreamHandler();
		}
	}


class DummyURLStreamHandler extends URLStreamHandler
	{
	/*
	A URL stream handler for all protocols, used to return our
	dummy implementation of URLConnection to open up a local
	file when called upon.
	*/

	protected final URLConnection openConnection(URL url)
		throws IOException
		{
		return new DummyURLConnection(url);
		}

	}


class DummyURLConnection extends URLConnection
	{
	/*
	Our dummy implementation of URLConnection used to open up a local
	file when called upon with a given URL of ANY protocol type. This
	allows the applet to easily use the "getInputStream()" function.
	*/

	boolean connected = false;
	InputStream instream;

	protected DummyURLConnection(URL url)
		{
		super(url);
		}

	public void connect() throws IOException
		{
		/*
		Open the local file.
		*/

		if (! connected)
			{
			String filename = url.getFile();
			if (filename.charAt(1) == '|')
				{
				StringBuffer buf = new StringBuffer(filename);
				buf.setCharAt(1, ':');
				filename = buf.toString();
				}
			else
				if (filename.charAt(2) == '|')
					{
					StringBuffer buf = new StringBuffer(filename);
					buf.setCharAt(2, ':');
					filename = buf.toString();
					}

			instream = new FileInputStream(filename);
			}
		}

	public InputStream getInputStream() throws IOException
		{
		/*
		Return the open stream to the local file (open if necessary).
		*/

		if (! connected)
			connect();

		if (instream == null)
			throw new IOException();

		return instream;
		}
	}
