
class ID
	{
	/*
	Define objects that can hold unique ids for a collection of objects
	of different classes.
	
	Public methods:
	---------------
	ID getID()          return a unique id
	boolean equals()    tell whether two ids are equal
	boolean notEquals() tell whether two ids are nonequal
	String toString()

	Note: Each getID() request returns an object wrapped around a unique
	private integer.
	*/

	private static long currentIDNumber = 0;
	private long id;

	//used to synchonize access to currentIDNumber to make it thread-safe
	private static Object monitor = new Object();

	/////////////////////////////////////////////////////////////////////
	private ID()
		{
		id = currentIDNumber;
		}

	/////////////////////////////////////////////////////////////////////
	public static ID getID()
		{
		synchronized(monitor)
			{
			currentIDNumber++;
			return new ID();
			}
		}

	/////////////////////////////////////////////////////////////////////
	public boolean equals(Object id)
		{
		if (! (id instanceof ID))
			return false;

		//check the superclass portions first (this is why i use Object above)
		//this should never fire, but it's here just in case i later extend ID
		//from some other class besides Object and forget to change this code
		if (super.equals(id) == false)
			return false;

		return this == id;
		}

	/////////////////////////////////////////////////////////////////////
	public boolean notEquals(ID id)
		{
		return (! this.equals(id));
		}

	/////////////////////////////////////////////////////////////////////
	public String toString()
		{
		return "" + id;
		}
	}
