
import java.util.Random;

class Rand
	{
	/*
	Generate random integers in a symmetric inclusive range.

	Public Method:
	--------------
	Rand.integer()

	Notes:
	(1) Random.nextFloat() returns a random float x where 0 <= x < 1. This
	class treats both ends uniformly so users don't have to remember the
	asymmetry.
	(2) randomNumberStream is initialized statically to ensure that the seed
	is sufficiently random---which means accessing hashCode()---which means
	creating an object. It's static so that user objects don't have to create
	an object just to get random numbers; they can just call a static method.
	*/

	private final static Rand singleton = new Rand();
	private final static Random randomNumberStream =
		new Random(singleton.hashCode() * System.currentTimeMillis());

	////////////////////////////////////////////////////////////////////
	public static int integer(double lower, double upper)
		{
		/*
		Return a random integer in the range lower..upper inclusive.
		If lower > upper, return an integer in the range upper..lower.
		*/

		double min = Math.min(lower, upper);
		double max = Math.max(lower, upper);

		return (int) min +
			(int) (randomNumberStream.nextFloat() * (max - min + 1));
		}
	}
