
import IllegalRectangleArgumentsException;

class Rectangle
	{
	/*
	Define objects that can store and interrogate a rectangle.

	Public Methods:
	---------------
	int getXMin()
	int getXMax()
	int getYMin()
	int getYMax()
	int getWidth()
	int getHeight()
	int getPerimeter()
	int getArea()

	boolean surrounds()      tell whether i surround another rectangle
	boolean isSurroundedBy() tell whether i'm surrounded by another rectangle
	boolean intersects()     tell whether i intersect another rectangle
	boolean xIntersect()     tell whether my x-range intersects another's
	boolean yIntersect()     tell whether my y-range intersects another's
	
	toString()

	Note: Java window coordinates place the origin at the upper-lefthand
	corner and x increases left-to-right, y increases top-to-bottom; so
	the upper-left corner is (xMin, yMin), the lower-left is (xMax, yMax).
	*/

	//the rectangle's basic variables
	protected int xMin, yMin, xMax, yMax;
	protected int width, height, perimeter, area;

	///////////////////////////////////////////////////////////////////
	public Rectangle(int xMin, int yMin, int width, int height)
		throws IllegalRectangleArgumentsException
		{
		this.xMin = xMin; this.yMin = yMin;
		this.xMax = xMin + width; this.yMax = yMin + height;
		this.width = width; this.height = height;
		perimeter = 2 * (width + height); area = width * height;

		if ((width < 0) || (height < 0))
			throw new IllegalRectangleArgumentsException();
		}

	///////////////////////////////////////////////////////////////////
	public int getXMin()
		{
		/*
		Return my lefthand (and smallest) x value.
		*/

		return xMin;
		}

	///////////////////////////////////////////////////////////////////
	public int getXMax()
		{
		/*
		Return my righthand (and largest) x value.
		*/

		return xMax;
		}

	///////////////////////////////////////////////////////////////////
	public int getYMin()
		{
		/*
		Return my upper (and smallest) y value.
		*/

		return yMin;
		}

	///////////////////////////////////////////////////////////////////
	public int getYMax()
		{
		/*
		Return my lower (and largest) y value.
		*/

		return yMax;
		}

	///////////////////////////////////////////////////////////////////
	public int getWidth()
		{
		/*
		Return my width.
		*/

		return width;
		}

	///////////////////////////////////////////////////////////////////
	public int getHeight()
		{
		/*
		Return my height.
		*/

		return height;
		}

	///////////////////////////////////////////////////////////////////
	public int getPerimeter()
		{
		/*
		Return my perimeter.
		*/

		return perimeter;
		}

	///////////////////////////////////////////////////////////////////
	public int getArea()
		{
		/*
		Return my area.
		*/

		return area;
		}

	//////////////////////////////////////////////////////////////////
	public boolean isSurroundedBy(Rectangle rectangle)
		{
		/*
		Check whether i'm inside another rectangle.
		*/

		if ((this.xMin < rectangle.getXMin()) ||
			(this.xMax > rectangle.getXMax()) ||
			(this.yMin < rectangle.getYMin()) ||
			(this.yMax > rectangle.getYMax()))
			return false;
		else
			return true;
		}

	//////////////////////////////////////////////////////////////////
	public boolean surrounds(Rectangle rectangle)
		{
		/*
		Check whether i surround rectangle.
		*/

		return rectangle.isSurroundedBy(this);
		}

	//////////////////////////////////////////////////////////////////
	public boolean xIntersect(Rectangle rectangle)
		{
		/*
		Check whether my x-range intersects rectangle's x-range.
		*/

		if ((rectangle.getXMin() <= this.xMax) &&
			(this.xMin <= rectangle.getXMax()))
			return true;
		else
			return false;
		}

	//////////////////////////////////////////////////////////////////
	public boolean yIntersect(Rectangle rectangle)
		{
		/*
		Check whether my y-range intersects rectangle's y-range.
		*/

		if ((rectangle.getYMin() <= this.yMax) &&
			(this.yMin <= rectangle.getYMax()))
			return true;
		else
			return false;
		}

	//////////////////////////////////////////////////////////////////
	public boolean intersects(Rectangle rectangle)
		{
		/*
		Check whether i intersect another rectangle.
		*/

		if (this.xIntersect(rectangle) && this.yIntersect(rectangle))
			return true;
		else
			return false;
		}

	///////////////////////////////////////////////////////////////////
	public String toString()
		{
		return "Rectangle[(" + xMin + "," + yMin + "), " +
			"(" + xMax + "," + yMax + ")]";
		}
	}
