
import Spawnable;
import HittableRectangle;
import IllegalRectangleArgumentsException;
import RectangleTooSmallToSpawnException;
import Rand;

class SpawnableRectangle extends HittableRectangle
	implements Spawnable
	{
	/*
	Define objects that can create new rectangles from old rectangles.
	Any spawned rectangles are contained within the spawning rectangle.
	*/

	//the minimum possible dimensions of any spawn
	protected static final int MIN_SPAWN_WIDTH  = 2;
	protected static final int MIN_SPAWN_HEIGHT = 2;

	//the minimum possible dimensions to be able to spawn
	protected static final int MIN_SPAWNABLE_WIDTH  = 4;
	protected static final int MIN_SPAWNABLE_HEIGHT = 4;

	///////////////////////////////////////////////////////////////////
	public SpawnableRectangle(int xMin, int yMin, int width, int height)
		throws IllegalRectangleArgumentsException
		{
		super(xMin, yMin, width, height);
		}

	//////////////////////////////////////////////////////////////////
	public Spawnable spawn()
		throws RectangleTooSmallToSpawnException
		{
		/*
		Return a random rectangle contained within my boundaries, if i'm big
		enough to spawn.
		*/

		if ((width < MIN_SPAWNABLE_WIDTH) ||
			(height < MIN_SPAWNABLE_HEIGHT))
			throw new RectangleTooSmallToSpawnException();

		//try to spawn until i produce a big enough spawn
		int newXMin = 0, newXMax = 0, newYMin = 0, newYMax = 0;
		while (((newXMax - newXMin) < MIN_SPAWN_WIDTH) ||
				(newYMax - newYMin) < MIN_SPAWN_HEIGHT)
			{
			//generate a random upper-left corner
			newXMin = Rand.integer(xMin, xMax - MIN_SPAWN_WIDTH);
			newYMin = Rand.integer(yMin, yMax - MIN_SPAWN_HEIGHT);

			//and a random lower-right corner
			newXMax = Rand.integer(newXMin, xMax);
			newYMax = Rand.integer(newYMin, yMax);
			}

		Spawnable spawn = null;
		try
			{
			spawn = new SpawnableRectangle(newXMin, newYMin,
													newXMax - newXMin, newYMax - newYMin);
			}
		catch (IllegalRectangleArgumentsException exception)
			{ //should never happen!
			System.out.println("SpawnableRectangle: internal error: " +
										exception + " Aborting.");
			System.exit(1);
			}

		return spawn;
		}
	}
